package com.pgpool.tests;


import com.pgpool.domain.AuthData;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.util.Random;

public class Issue271_SingleThread extends PgpoolTest {


	@Override
	public void runTest(AuthData authData) throws Exception {

		String url = String.format("jdbc:postgresql://%s:%d/%s?loglevel=2", authData.getPgpoolHostname(), authData.getPgpoolPort(), authData.getDatabaseName());

		Random random = new Random(System.currentTimeMillis());

		try (Connection conn = DriverManager.getConnection(url, authData.getDatabaseLogin(), authData.getDatabasePasswd())) {
			for (int j = 0; j < 1; j++) { // TODO: move to parameters

				conn.setAutoCommit(false); // here we define explicit transaction

				int testValue = random.nextInt();

				try (PreparedStatement stmtInsert = conn.prepareStatement("select id from write_table where id = ?")) {
					stmtInsert.setInt(1, testValue);
					for (int i = 0; i < 20; i++) {

						try (ResultSet rs = stmtInsert.executeQuery()) {
							if (rs.next())
								throw new RuntimeException("This should not happen, please empty the 'write_table' and restart trhe test");
						}
					}
				}
				conn.commit();

				try (PreparedStatement stmtInsert = conn.prepareStatement("insert into write_table(id) values(?)")) {
					stmtInsert.setInt(1, testValue);
					stmtInsert.execute();
					//System.out.println("Value inserted");
				}

				try (PreparedStatement stmtInsert = conn.prepareStatement("select id from write_table where id = ?")) {
					stmtInsert.setInt(1, testValue);
					for (int i = 0; i < 20; i++) {

						if (!stmtInsert.executeQuery().next()) {
							throw new RuntimeException("Balanced");
						}
					}
				}
				conn.commit();
			}
		}
	}
}
